package xyz.lychee.lagfixer.libs.adventure.platform.bukkit;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import java.lang.invoke.MethodHandle;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.serializer.ComponentSerializer;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.Experimental;

@Experimental
public final class MinecraftComponentSerializer implements ComponentSerializer<Component, Component, Object> {
   private static final MinecraftComponentSerializer INSTANCE = new MinecraftComponentSerializer();
   private static final @Nullable Class<?> CLASS_JSON_DESERIALIZER = MinecraftReflection.findClass("com.goo".concat("gle.gson.JsonDeserializer"));
   private static final @Nullable Class<?> CLASS_CHAT_COMPONENT = MinecraftReflection.findClass(MinecraftReflection.findNmsClassName("IChatBaseComponent"), MinecraftReflection.findMcClassName("network.chat.IChatBaseComponent"), MinecraftReflection.findMcClassName("network.chat.Component"));
   private static final AtomicReference<RuntimeException> INITIALIZATION_ERROR = new AtomicReference(new UnsupportedOperationException());
   private static final Object MC_TEXT_GSON;
   private static final MethodHandle TEXT_SERIALIZER_DESERIALIZE;
   private static final MethodHandle TEXT_SERIALIZER_SERIALIZE;
   private static final MethodHandle TEXT_SERIALIZER_DESERIALIZE_TREE;
   private static final MethodHandle TEXT_SERIALIZER_SERIALIZE_TREE;
   private static final boolean SUPPORTED;

   public static boolean isSupported() {
      return SUPPORTED;
   }

   public static @NotNull MinecraftComponentSerializer get() {
      return INSTANCE;
   }

   public @NotNull Component deserialize(final @NotNull Object input) {
      if (!SUPPORTED) {
         throw (RuntimeException)INITIALIZATION_ERROR.get();
      } else {
         try {
            JsonElement element;
            if (TEXT_SERIALIZER_SERIALIZE_TREE != null) {
               element = TEXT_SERIALIZER_SERIALIZE_TREE.invoke(input);
            } else {
               if (MC_TEXT_GSON == null) {
                  return BukkitComponentSerializer.gson().deserialize(TEXT_SERIALIZER_SERIALIZE.invoke(input));
               }

               element = ((Gson)MC_TEXT_GSON).toJsonTree(input);
            }

            return (Component)BukkitComponentSerializer.gson().serializer().fromJson(element, Component.class);
         } catch (Throwable var3) {
            Throwable error = var3;
            throw new UnsupportedOperationException(error);
         }
      }
   }

   public @NotNull Object serialize(final @NotNull Component component) {
      if (!SUPPORTED) {
         throw (RuntimeException)INITIALIZATION_ERROR.get();
      } else if (TEXT_SERIALIZER_DESERIALIZE_TREE == null && MC_TEXT_GSON == null) {
         try {
            return TEXT_SERIALIZER_DESERIALIZE.invoke((String)BukkitComponentSerializer.gson().serialize(component));
         } catch (Throwable var4) {
            Throwable error = var4;
            throw new UnsupportedOperationException(error);
         }
      } else {
         JsonElement json = BukkitComponentSerializer.gson().serializer().toJsonTree(component);

         try {
            return TEXT_SERIALIZER_DESERIALIZE_TREE != null ? TEXT_SERIALIZER_DESERIALIZE_TREE.invoke(json) : ((Gson)MC_TEXT_GSON).fromJson(json, CLASS_CHAT_COMPONENT);
         } catch (Throwable var5) {
            Throwable error = var5;
            throw new UnsupportedOperationException(error);
         }
      }
   }

   static {
      Object gson = null;
      MethodHandle textSerializerDeserialize = null;
      MethodHandle textSerializerSerialize = null;
      MethodHandle textSerializerDeserializeTree = null;
      MethodHandle textSerializerSerializeTree = null;

      try {
         if (CLASS_CHAT_COMPONENT != null) {
            Class<?> chatSerializerClass = (Class)Arrays.stream(CLASS_CHAT_COMPONENT.getClasses()).filter((c) -> {
               if (CLASS_JSON_DESERIALIZER != null) {
                  return CLASS_JSON_DESERIALIZER.isAssignableFrom(c);
               } else {
                  Class[] var1 = c.getInterfaces();
                  int var2 = var1.length;

                  for(int var3 = 0; var3 < var2; ++var3) {
                     Class<?> itf = var1[var3];
                     if (itf.getSimpleName().equals("JsonDeserializer")) {
                        return true;
                     }
                  }

                  return false;
               }
            }).findAny().orElse(MinecraftReflection.findNmsClass("ChatSerializer"));
            if (chatSerializerClass != null) {
               Field gsonField = (Field)Arrays.stream(chatSerializerClass.getDeclaredFields()).filter((m) -> {
                  return Modifier.isStatic(m.getModifiers());
               }).filter((m) -> {
                  return m.getType().equals(Gson.class);
               }).findFirst().orElse((Object)null);
               if (gsonField != null) {
                  gsonField.setAccessible(true);
                  gson = gsonField.get((Object)null);
               }
            }

            List<Class<?>> candidates = new ArrayList();
            if (chatSerializerClass != null) {
               candidates.add(chatSerializerClass);
            }

            candidates.addAll(Arrays.asList(CLASS_CHAT_COMPONENT.getClasses()));
            Iterator var7 = candidates.iterator();

            while(var7.hasNext()) {
               Class<?> serializerClass = (Class)var7.next();
               Method[] declaredMethods = serializerClass.getDeclaredMethods();
               Method deserialize = (Method)Arrays.stream(declaredMethods).filter((m) -> {
                  return Modifier.isStatic(m.getModifiers());
               }).filter((m) -> {
                  return CLASS_CHAT_COMPONENT.isAssignableFrom(m.getReturnType());
               }).filter((m) -> {
                  return m.getParameterCount() == 1 && m.getParameterTypes()[0].equals(String.class);
               }).min(Comparator.comparing(Method::getName)).orElse((Object)null);
               Method serialize = (Method)Arrays.stream(declaredMethods).filter((m) -> {
                  return Modifier.isStatic(m.getModifiers());
               }).filter((m) -> {
                  return m.getReturnType().equals(String.class);
               }).filter((m) -> {
                  return m.getParameterCount() == 1 && CLASS_CHAT_COMPONENT.isAssignableFrom(m.getParameterTypes()[0]);
               }).findFirst().orElse((Object)null);
               Method deserializeTree = (Method)Arrays.stream(declaredMethods).filter((m) -> {
                  return Modifier.isStatic(m.getModifiers());
               }).filter((m) -> {
                  return CLASS_CHAT_COMPONENT.isAssignableFrom(m.getReturnType());
               }).filter((m) -> {
                  return m.getParameterCount() == 1 && m.getParameterTypes()[0].equals(JsonElement.class);
               }).findFirst().orElse((Object)null);
               Method serializeTree = (Method)Arrays.stream(declaredMethods).filter((m) -> {
                  return Modifier.isStatic(m.getModifiers());
               }).filter((m) -> {
                  return m.getReturnType().equals(JsonElement.class);
               }).filter((m) -> {
                  return m.getParameterCount() == 1 && CLASS_CHAT_COMPONENT.isAssignableFrom(m.getParameterTypes()[0]);
               }).findFirst().orElse((Object)null);
               if (deserialize != null) {
                  textSerializerDeserialize = MinecraftReflection.lookup().unreflect(deserialize);
               }

               if (serialize != null) {
                  textSerializerSerialize = MinecraftReflection.lookup().unreflect(serialize);
               }

               if (deserializeTree != null) {
                  textSerializerDeserializeTree = MinecraftReflection.lookup().unreflect(deserializeTree);
               }

               if (serializeTree != null) {
                  textSerializerSerializeTree = MinecraftReflection.lookup().unreflect(serializeTree);
               }
            }
         }
      } catch (Throwable var14) {
         Throwable error = var14;
         INITIALIZATION_ERROR.set(new UnsupportedOperationException("Error occurred during initialization", error));
      }

      MC_TEXT_GSON = gson;
      TEXT_SERIALIZER_DESERIALIZE = textSerializerDeserialize;
      TEXT_SERIALIZER_SERIALIZE = textSerializerSerialize;
      TEXT_SERIALIZER_DESERIALIZE_TREE = textSerializerDeserializeTree;
      TEXT_SERIALIZER_SERIALIZE_TREE = textSerializerSerializeTree;
      SUPPORTED = MC_TEXT_GSON != null || TEXT_SERIALIZER_DESERIALIZE != null && TEXT_SERIALIZER_SERIALIZE != null || TEXT_SERIALIZER_DESERIALIZE_TREE != null && TEXT_SERIALIZER_SERIALIZE_TREE != null;
   }
}
